@@ -53,6 +53,7 @@ def docs():
         "[26] ssap.scatter02(df, parameter_x, parameter_y, parameter_z, bubblesize, title)\n"
         "[27] ssap.scatter03(df, parameter_x, parameter_y, category, title)\n"
         "[28] ssap.map()\n"
+        "[29] ssap.map01(df, province, parameter_x, method)\n"
     )
     # explain details:
     # print(
@@ -972,10 +973,6 @@ def map():
     return m
 
 
-
-
-
-# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> [to be validated.]
 # map with value method(max,min,mean,median,sum,std_err)
 def map01(df, province, parameter_x, method):
     import matplotlib.pyplot as plt
@@ -1046,13 +1043,206 @@ def map01(df, province, parameter_x, method):
 
 
 
+# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> [to be validated. update:12/28]
+# scatter plot + hist plot on top/right.
+def scatter04hist(df, parameter_x, parameter_y, bins, title):
+    import matplotlib.pyplot as plt
+
+    df = df
+    x = df[parameter_x]
+    y = df[parameter_y]
+    title = title
+
+    # definitions for the axes
+    left, width = 0.1, 0.65
+    bottom, height = 0.1, 0.65
+    spacing = 0.005
+
+    rect_scatter = [left, bottom, width, height]
+    rect_histx = [left, bottom + height + spacing, width, 0.2]
+    rect_histy = [left + width + spacing, bottom, 0.2, height]
+
+    # start with a rectangular Figure
+    plt.figure(figsize=(8, 8))
+
+    # scatter plot
+    ax_scatter = plt.axes(rect_scatter)
+    plt.grid(linestyle='--', lw=0.5)
+    ax_scatter.tick_params(direction='in', top=True, right=True)
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+
+    # top hist
+    ax_histx = plt.axes(rect_histx)
+    plt.grid(linestyle='--', lw=0.5, axis='y')
+    ax_histx.tick_params(direction='in', labelbottom=False)
+    plt.xlabel(parameter_x)
+    plt.ylabel('Counts')
+    plt.title(title)
+
+    # right hist
+    ax_histy = plt.axes(rect_histy)
+    plt.grid(linestyle='--', lw=0.5, axis='x')
+    ax_histy.tick_params(direction='in', labelleft=False)
+    plt.xlabel('Counts')
+    plt.ylabel(parameter_y)
+
+    # the scatter plot:
+    ax_scatter.scatter(x, y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='darkorange')
+
+    bins = bins
+
+    # now determine nice limits by hand:
+    ax_histx.hist(x, bins=bins, edgecolor='black', facecolor='darkorange', alpha=0.7)
+    ax_histy.hist(y, bins=bins, orientation='horizontal', edgecolor='black', facecolor='darkorange', alpha=0.7)
+
+    ax_histx.set_xlim(ax_scatter.get_xlim())
+    #     ax_histy.set_ylim(ax_scatter.get_ylim())
+
+    plt.show()
+
+
+
+# show correlation line + error zone. with scatter plot:
+
+
+# show colormap detail:
+def colormap():
+    import numpy as np
+    import matplotlib.pyplot as plt
+
+
+    cmaps = [('Perceptually Uniform Sequential', [
+                'viridis', 'plasma', 'inferno', 'magma', 'cividis']),
+             ('Sequential', [
+                'Greys', 'Purples', 'Blues', 'Greens', 'Oranges', 'Reds',
+                'YlOrBr', 'YlOrRd', 'OrRd', 'PuRd', 'RdPu', 'BuPu',
+                'GnBu', 'PuBu', 'YlGnBu', 'PuBuGn', 'BuGn', 'YlGn']),
+             ('Sequential (2)', [
+                'binary', 'gist_yarg', 'gist_gray', 'gray', 'bone', 'pink',
+                'spring', 'summer', 'autumn', 'winter', 'cool', 'Wistia',
+                'hot', 'afmhot', 'gist_heat', 'copper']),
+             ('Diverging', [
+                'PiYG', 'PRGn', 'BrBG', 'PuOr', 'RdGy', 'RdBu',
+                'RdYlBu', 'RdYlGn', 'Spectral', 'coolwarm', 'bwr', 'seismic']),
+             ('Cyclic', ['twilight', 'twilight_shifted', 'hsv']),
+             ('Qualitative', [
+                'Pastel1', 'Pastel2', 'Paired', 'Accent',
+                'Dark2', 'Set1', 'Set2', 'Set3',
+                'tab10', 'tab20', 'tab20b', 'tab20c']),
+             ('Miscellaneous', [
+                'flag', 'prism', 'ocean', 'gist_earth', 'terrain', 'gist_stern',
+                'gnuplot', 'gnuplot2', 'CMRmap', 'cubehelix', 'brg',
+                'gist_rainbow', 'rainbow', 'jet', 'nipy_spectral', 'gist_ncar'])]
+
+
+    gradient = np.linspace(0, 1, 256)
+    gradient = np.vstack((gradient, gradient))
+
+
+    def plot_color_gradients(cmap_category, cmap_list):
+        # Create figure and adjust figure height to number of colormaps
+        nrows = len(cmap_list)
+        figh = 0.35 + 0.15 + (nrows + (nrows-1)*0.1)*0.22
+        fig, axes = plt.subplots(nrows=nrows, figsize=(6.4, figh))
+        fig.subplots_adjust(top=1-.35/figh, bottom=.15/figh, left=0.2, right=0.99)
+
+        axes[0].set_title(cmap_category + ' colormaps', fontsize=14)
+
+        for ax, name in zip(axes, cmap_list):
+            ax.imshow(gradient, aspect='auto', cmap=plt.get_cmap(name))
+            ax.text(-.01, .5, name, va='center', ha='right', fontsize=10,
+                    transform=ax.transAxes)
+
+        # Turn off *all* ticks & spines, not just the ones with colormaps.
+        for ax in axes:
+            ax.set_axis_off()
+
+
+    for cmap_category, cmap_list in cmaps:
+        plot_color_gradients(cmap_category, cmap_list)
+
+    plt.show()
+
+
+# show colorbar detail:
+def colorbar():
+
+    import matplotlib.pyplot as plt
+    import matplotlib.colors as mcolors
+
+
+    def plot_colortable(colors, title, sort_colors=True, emptycols=0):
+
+        cell_width = 212
+        cell_height = 22
+        swatch_width = 48
+        margin = 12
+        topmargin = 40
+
+        # Sort colors by hue, saturation, value and name.
+        if sort_colors is True:
+            by_hsv = sorted((tuple(mcolors.rgb_to_hsv(mcolors.to_rgb(color))),
+                             name)
+                            for name, color in colors.items())
+            names = [name for hsv, name in by_hsv]
+        else:
+            names = list(colors)
+
+        n = len(names)
+        ncols = 4 - emptycols
+        nrows = n // ncols + int(n % ncols > 0)
+
+        width = cell_width * 4 + 2 * margin
+        height = cell_height * nrows + margin + topmargin
+        dpi = 72
+
+        fig, ax = plt.subplots(figsize=(width / dpi, height / dpi), dpi=dpi)
+        fig.subplots_adjust(margin/width, margin/height,
+                            (width-margin)/width, (height-topmargin)/height)
+        ax.set_xlim(0, cell_width * 4)
+        ax.set_ylim(cell_height * (nrows-0.5), -cell_height/2.)
+        ax.yaxis.set_visible(False)
+        ax.xaxis.set_visible(False)
+        ax.set_axis_off()
+        ax.set_title(title, fontsize=24, loc="left", pad=10)
+
+        for i, name in enumerate(names):
+            row = i % nrows
+            col = i // nrows
+            y = row * cell_height
+
+            swatch_start_x = cell_width * col
+            swatch_end_x = cell_width * col + swatch_width
+            text_pos_x = cell_width * col + swatch_width + 7
+
+            ax.text(text_pos_x, y, name, fontsize=14,
+                    horizontalalignment='left',
+                    verticalalignment='center')
+
+            ax.hlines(y, swatch_start_x, swatch_end_x,
+                      color=colors[name], linewidth=18)
+
+        return fig
+
+    plot_colortable(mcolors.BASE_COLORS, "Base Colors",
+                    sort_colors=False, emptycols=1)
+    plot_colortable(mcolors.TABLEAU_COLORS, "Tableau Palette",
+                    sort_colors=False, emptycols=2)
+
+    #sphinx_gallery_thumbnail_number = 3
+    plot_colortable(mcolors.CSS4_COLORS, "CSS Colors")
+
+    # Optionally plot the XKCD colors (Caution: will produce large figure)
+    #xkcd_fig = plot_colortable(mcolors.XKCD_COLORS, "XKCD Colors")
+    #xkcd_fig.savefig("XKCD_Colors.png")
+
 
 # [planning ↓]
 
 
 
-
-# heatmap()
+# heatmap() / correlation map
 # Line plot (x-时间序列: 4.移动平均图 Arima  5.时间序列预测 等种种）
 # my.data analytics ppt里的图表
 # 多因子相关性矩阵图/
